class MeiToMesController < ApplicationController
  include MachineResponse
  include MachineTranslator

  before_action :resolve_machine_target

  class NoTranslationInterface < StandardError; end
  class MissingRequiredParam < StandardError; end
  class InvalidMachineState < StandardError; end
  class InvalidLensSide < StandardError; end
  class InvalidJobState < StandardError; end

  def machine_status_add
    begin
      params = machine_status_params
      model_params = MachineTranslator.get_translator(@machine.translator_class)
                                      .get_translation(params, MachineTranslator.machine_state_type)
      MachineInterfaceJobs::MachineStateInterface::AddMachineStateJob.perform_later(model_params)
    rescue => e
      Rails.logger.error "Error in adding Machine State from MEI - #{e}"
    end
    send_machine_response
  end

  def job_status_add
    begin
      params = job_status_params
      model_params = MachineTranslator.get_translator(@machine.translator_class)
                                      .get_translation(params, MachineTranslator.job_state_type)
      MachineInterfaceJobs::JobStateInterface::AddJobStateJob.perform_later(model_params)
    rescue => e
      Rails.logger.error "Error in adding Job State from MEI - #{e}"
    end
    send_machine_response
  end

  def alarm_add
    begin
      params = alarm_params
      model_params = MachineTranslator.get_translator(@machine.translator_class)
                                      .get_translation(params, MachineTranslator.error_type)
      MachineInterfaceJobs::ErrorInterface::AddErrorJob.perform_later(model_params)
    rescue => e
      Rails.logger.error "Error in adding Alarm from MEI - #{e}"
    end
    send_machine_response
  end

  def quality_control_add
  #   Potential endpoint for quality control
  end

  private

  def resolve_machine_target
    return render json: {error: 'missing machine sn'}, status: 422 if params[:sn].blank?
    @machine = Machine.find_or_create_by(sn: params[:sn])

    # To use the MEI translator API, the machine must be defined have a type before we hit the target endpoint
    if @machine.translator_class.nil?
      Rails.logger.warn "Cannot to convert MEI data to MES - machine #{@machine.sn} has no type"
      return send_machine_response
    end
  end

  def base_params
    {
      machine_sn: params[:sn],
      time: "#{params[:date]} #{params[:time]}",
    }
  end

  def machine_status_params
    machine_state = params.require(:MtmpMachineStatus)
    status = machine_state[:Status].downcase

    allowed_machine_states = %w[working workingidle on notready ready alarm off]
    raise InvalidMachineState if allowed_machine_states.exclude?(status)

    base_params.merge! (
                          {
                            machine_state: {
                              state: status,
                            }
                          }
                       )
  end

  def job_status_params
    job_state_params = params.require(:MtmpJobStatus)

    lens_side = job_state_params[:LensSide].downcase
    status = job_state_params[:Status].downcase

    allowed_lenssides = %w[r l unknown]
    allowed_job_states = %w[started completed aborted]
    raise InvalidLensSide if allowed_lenssides.exclude?(lens_side)
    raise InvalidJobState if allowed_job_states.exclude?(status)

    # Default 'unknown' lens side to left
    lens_side = "l" if lens_side == "unknown"

    base_params.merge! ({
      job: {
        job_number: job_state_params[:Job],
        side: lens_side
      },
      job_state: {
        state: status,
        process: job_state_params[:Process],
        comment: job_state_params[:Comment]
      }
    })
  end

  def alarm_params
    error = params.require(:MtmpAlarm)

    base_params.merge! ({
      job: nil,
      error: {
        piececounter: 0,
        errno: 0,
        error: error[:text],
        errortype: 2,
        status: 0
      }
    })
  end

  def quality_control_params
    # TODO: format process params here
  end
end
