module MachineInterfaceJobs
  module JobProcessInterface
    module Computed
      class GenerateConsumableRecord < ApplicationJob

        def perform(job_process)
          @job_process = job_process

          return unless @job_process.machine.collects_consumable_data?

          @machine_type = @job_process.machine.machine_type

          @consumable_process_parameters = extract_consumable_process_params

          # Machines should always send all available keys even for unused tools (MT=0 for example)
          return if @consumable_process_parameters.keys.count != @machine_type.consumable_data_params[:keys].count

          @machine_type.consumable_data_params[:consumables].each do |consumable_config|
            next unless consumable_is_in_use(consumable_config[:enabled_key])

            consumable_name = consumable_config[:resolved_name].call(@consumable_process_parameters)

            next unless record_in_chronological_order(consumable_name)

            existing_consumable_record = ConsumableRecord.find_by(name: consumable_name, machine_id: @job_process.machine_id, retired: false)

            if existing_consumable_record
              if is_new_consumable(consumable_config[:use_count_key], existing_consumable_record.current_use_count)
                JobProcess.transaction do
                  existing_consumable_record.update(retired: true)
                  new_consumable_record(consumable_name, consumable_config[:use_count_key], consumable_config[:max_use_count_key], consumable_config[:data])
                end
              else
                update_consumable_record(existing_consumable_record, consumable_config[:use_count_key], consumable_config[:data])
              end
            else
              new_consumable_record(consumable_name, consumable_config[:use_count_key], consumable_config[:max_use_count_key], consumable_config[:data])
            end
          end
        end

        private

        def extract_consumable_process_params
          process_params = @job_process.job_process_parameters
                                       .where(JobProcessParameter.arel_table[:name].in(@machine_type.consumable_data_params[:keys]))
                                       .pluck(:name, :value)

          process_params.to_h.symbolize_keys
        end

        def new_consumable_record(consumable_name, use_count_key, max_use_count_key, consumable_config_data)
          new_record = ConsumableRecord.new(
            name: consumable_name,
            current_use_count: @consumable_process_parameters[use_count_key],
            max_use_count: @consumable_process_parameters[max_use_count_key],
            first_use_date: @job_process.time,
            last_use_date: @job_process.time,
            retired: false,
            machine_id: @job_process.machine_id
          )

          consumable_config_data.each do |consumable_datum|
            new_record.consumable_record_data.build(
              name: consumable_datum[:name],
              value: get_consumable_datum_value(consumable_datum)
            )
          end

          new_record.save!
        end

        def update_consumable_record(existing_consumable_record, use_count_key, consumable_config_data)
          ConsumableRecord.transaction do
            existing_consumable_record.assign_attributes(current_use_count: @consumable_process_parameters[use_count_key], last_use_date: @job_process.time)

            consumable_config_data.filter { |i| i[:touch_on_update] }.each do |consumable_datum|
              existing_consumable_record.consumable_record_data.find_by(name: consumable_datum[:name]).update!(value: get_consumable_datum_value(consumable_datum))
            end

            existing_consumable_record.save!
          end
        end

        def consumable_is_in_use(enabled_key)
          @consumable_process_parameters[enabled_key].to_i != 0
        end

        def is_new_consumable(use_count_key, consumable_record_value)
          @consumable_process_parameters[use_count_key].to_i < consumable_record_value
        end

        def get_consumable_datum_value(consumable_datum)
          return consumable_datum[:compute].call(@consumable_process_parameters) unless consumable_datum[:compute].nil?
          @consumable_process_parameters[consumable_datum[:key]]
        end

        def record_in_chronological_order(consumable_name)
          last = ConsumableRecord.find_by(name: consumable_name, machine_id: @job_process.machine_id, retired: false)
          return true if last.nil?
          @job_process.time > last.last_use_date
        end
      end
    end
  end
end